require 'class_extensions'

class OsmGuidepost < ActiveRecord::Base
  has_many :osm_signs, :dependent => :destroy # hazelot divne chyby pri destroy
  has_many :osm_trails, :through => :osm_signs
  has_one :guidepost, :foreign_key => :osmid, :primary_key=> :osmid
  has_many :photos, inverse_of: :osm_guidepost, foreign_key: :osmid, primary_key: :osmid
  scope :search, lambda { |term| where("CONCAT_WS('', kctref, LOWER(nazev)) LIKE ?", "%#{term.mb_chars.downcase
}%")  }
  
  serialize :json, Hash
  include CommonGuidepost

  def sez_guidepost; guidepost.andand.sez_guidepost end  
  def kct_guidepost; guidepost.andand.kct_guidepost end
  def kct_guidepost_by_kctref; KctGuidepost.find_by_kctref(kctref) end
  def importable?; !kct_guidepost_by_kctref.nil? end
  def osm_guidepost; self end

  def self.new_from_json(ajson)
    result = OsmGuidepost.new
    result.initialize_from_json(ajson)
    return result
  end
  
  # v případě že guidepost existuje, tak ho jen updateuje
  def self.new_or_update_from_json(ajson)
    # find or create
    result = OsmGuidepost.find_by_osmid(ajson['id'])
    if !result.nil?
      result.initialize_from_json(ajson)
      return result
    else
      return OsmGuidepost.new_from_json(ajson)
    end
  end
  
  def initialize_from_json(ajson)
    self.json = ajson
    self.osmid = self.json['id']
    self.nazev = self.json['tags']['name']
    self.nazev_normalized = nazev.normalize unless nazev.nil?
    self.lat = self.json['lat']
    self.lon = self.json['lon']
    self.kctref = self.json['tags']['ref'] || self.json['tags']['ref:kct']
    self.vyska = self.json['tags']['ele']
  end
  
  ## accessing data
  def json_other
    json['tags'].keys.reject{|k| ['name', 'ele','hiking','information','ref','tourism','created_by'].include?(k)}.
      collect{|key| "#{key} = #{json['tags'][key]}"}.join(', ')
  end
  def coordinates; [lat,lon] end
  
  def self.where_kctref_not_empty
    OsmGuidepost.all.where('(kctref != "") AND (kctref IS NOT NULL)')
  end
  
  ## finding similar Guideposts
  def unikatni_dle_kctref_v_osm_guideposts?
    OsmGuidepost.where(:kctref=>self.kctref).size == 1
  end
  def similar_to_existing?
    (distance_from_existing < 100) or
    self.exists_by_name? or
    self.exists_by_osmid?
  end
  def exists?;            !guidepost.nil? end
  def exists_by_name?;    Guidepost.find_by_name(self.name) rescue false end
  def exists_by_osmid?;   Guidepost.find_by_osmid(self.id) rescue false  end
  def exists_by_kctref?;  Guidepost.find_by_kctref(self.kctref) rescue false end
  def exists_by_any_ref?; exists_by_osmid? or exists_by_kctref? end
  def guidepost_candidate
    if exists_by_osmid?
      Guidepost.find_by_osmid(self.id)
    elsif exists_by_kctref?
      Guidepost.find_by_kctref(self.kctref)
    else
      nil
    end
  end

  def distance_from_existing
    mylon = self.lon; mylat = self.lat; 
    mindist = 999
    mindistid = nil
    Guidepost.all_with_coordinates.each do |gp|
      currdist = (mylon-gp.lon).abs*68420 + (mylat-gp.lat).abs*111268  # approx in meters
      if currdist < mindist
        mindist = currdist
        mindistid = gp.id
      end
    end
    mindist
  end
  def intent
    if self.similar_to_existing?
      'exist'
    else
      'does not exist in dbf'
    end
  end
  def inspect_nazev
    return nazev unless nazev.blank?
    return "Rozcestník KČT #{kctref}" unless kctref.blank?
    return "Rozcestník OSM #{osmid}" unless osmid.blank?
    'N/A'
  end
  def to_s(option=nil)
    case option
      when :long then "#{nazev} osm:#{osmid}"
      else "og:#{osmid}"
    end
  end
  
  # adds this to database
  def add_to_database
    Guidepost.new(:nazev => self.name, :kctref => self.ref, :lat => self.lat, :lon => self.lon, :osmid => self.id).save!
  end

  def self.symbol; '&#10499;'.html_safe end  
  def is_priority_osm_guidepost?
    !kctref.nil? and (!kctref.match /(P(S|J)\d\d\d)/)
  end
  def kctrefs
    self.kctref.andand.split(';') || []
  end

end



